<?php

class Platnosci_pl_model extends Model
{
	function Platnosci_pl_model()
	{
		parent::Model();
		$this->db->trans_strict(FALSE);
	}
	
	function transakcjaIstnieje($session_id=null)
	{
		$this->db->where('session_id',$session_id);
		$query = $this->db->get('platnosci_pl_transakcje');
		if (count($query->result()))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	function pobierzIdTransakcji($session_id=null)
	{
		$this->db->select('id_transakcji ');
		$this->db->where('session_id',$session_id);
		$query = $this->db->get('platnosci_pl_transakcje');
		$data = $query->result();
		if (count($data))
		{
			return $data[0]->id_transakcji;
		}
		else
		{
			return false;
		}
	}
	
	function pobierzIdFaktury($session_id=null)
	{
		$this->db->select('desc2');
		$this->db->where('session_id',$session_id);
		$query = $this->db->get('platnosci_pl_transakcje');
		$data = $query->result();
		if (count($data))
		{
			return $data[0]->desc2;
		}
		else
		{
			return false;
		}
	}
	
	function dodajNowaTransakcje($session_id=null)
	{		
		$this->db->trans_begin();
		$dane_transakcji['id_transakcji'] = '';
		$dane_transakcji['session_id'] = $session_id;
		$this->db->insert('platnosci_pl_transakcje',$dane_transakcji);
		$insert_id = $this->db->insert_id();
		//$this->db->trans_complete();
		if ($this->db->trans_status() === FALSE)
		{
			$this->db->trans_rollback();
			return false;
		}
		else
		{
			$this->db->trans_commit();
			//print $this->db->last_query();
			return $insert_id;
		}
	}
	
	function dodajTransakcje($dane_transakcji=null)
	{		
		$this->db->trans_begin();
		$dane_transakcji['id_transakcji'] = '';
		$this->db->insert('platnosci_pl_transakcje',$dane_transakcji);
		$insert_id = $this->db->insert_id();
		//$this->db->trans_complete();
		if ($this->db->trans_status() === FALSE)
		{
			$this->db->trans_rollback();
			return false;
		}
		else
		{
			$this->db->trans_commit();
			//print $this->db->last_query();
			return $insert_id;
		}
	}
	
	function zmienTransakcje($dane_transakcji=null)
	{
		$this->db->trans_begin();
		$this->db->where('session_id', $dane_transakcji['session_id']);
		$this->db->update('platnosci_pl_transakcje', $dane_transakcji);
		//$this->db->trans_complete();
		if ($this->db->trans_status() === FALSE)
		{
			$this->db->trans_rollback();
		}
		else
		{
			$this->db->trans_commit();
		}
	}
	
	function usunTransakcje($session_id=null)
	{
		$this->db->where('session_id', $session_id);
		$this->db->delete('platnosci_pl_transakcje'); 
	}
	
	function dodajLog($log=null)
	{
		$this->db->query("INSERT INTO platnosci_pl_log VALUES ('','$log', NOW());"); 
	}
}

?>